@@ -27,11 +27,12 @@
 
 #undef  SPI_XKB_DEBUG
 #undef  SPI_DEBUG
-#undef  SPI_KEYEVENT_DEBUG
+#define  SPI_KEYEVENT_DEBUG
 
 #include <string.h>
 #include <ctype.h>
 #include <stdio.h>
+#include <sys/time.h>
 #include <bonobo/bonobo-exception.h>
 
 #include <X11/Xlib.h>
@@ -185,11 +186,20 @@ spi_dec_set_unlatch_pending (SpiDEController *controller, unsigned mask)
   DEControllerPrivateData *priv = 
     g_object_get_qdata (G_OBJECT (controller), spi_dec_private_quark);
 #ifdef SPI_XKB_DEBUG
-  if (priv->xkb_latch_mask) fprintf (stderr, "unlatch pending! %x\n", mask);
+  if (priv->xkb_latch_mask) fprintf (stderr, "unlatch pending! %x\n", 
+				     priv->xkb_latch_mask);
 #endif
   priv->pending_xkb_mod_relatch_mask |= priv->xkb_latch_mask; 
 }
  
+static void
+spi_dec_clear_unlatch_pending (SpiDEController *controller)
+{
+  DEControllerPrivateData *priv = 
+    g_object_get_qdata (G_OBJECT (controller), spi_dec_private_quark);
+  priv->xkb_latch_mask = 0; 
+}
+ 
 static gboolean
 spi_dec_button_update_and_emit (SpiDEController *controller, 
 				guint mask_return)
@@ -445,6 +455,7 @@ spi_dec_poll_mouse_moving (gpointer data)
     }
 }
 
+#ifdef WE_NEED_UGRAB_MOUSE
 static int
 spi_dec_ungrab_mouse (gpointer data)
 {
@@ -456,6 +467,7 @@ spi_dec_ungrab_mouse (gpointer data)
 	  }
 	return FALSE;
 }
+#endif
 
 static void
 spi_dec_init_mouse_listener (SpiRegistry *registry)
@@ -465,10 +477,11 @@ spi_dec_init_mouse_listener (SpiRegistry *registry)
 
   if (display)
     {
-      XGrabButton (display, AnyButton, AnyModifier,
+      if (XGrabButton (display, AnyButton, AnyModifier,
 		   gdk_x11_get_default_root_xwindow (),
 		   True, ButtonPressMask | ButtonReleaseMask,
-		   GrabModeSync, GrabModeAsync, None, None);
+		   GrabModeSync, GrabModeAsync, None, None) != Success)
+	fprintf (stderr, "WARNING: could not grab mouse buttons!\n");
       XSync (display, False);
 #ifdef SPI_DEBUG
       fprintf (stderr, "mouse buttons grabbed\n");
@@ -808,7 +821,7 @@ spi_device_event_controller_forward_mouse_event (SpiDEController *controller,
   int button = xbutton_event->button;
   
   unsigned int mouse_button_state = xbutton_event->state;
-  
+
   switch (button)
     {
     case 1:
@@ -878,7 +891,7 @@ spi_device_event_controller_forward_mouse_event (SpiDEController *controller,
   /* if client wants to consume this event, and XKB latch state was
    *   unset by this button event, we reset it
    */
-  if (is_consumed && (xkb_mod_unlatch_occurred))
+  if (is_consumed && xkb_mod_unlatch_occurred)
     spi_dec_set_unlatch_pending (controller, mouse_mask_state);
   
   XAllowEvents (spi_get_display (),
@@ -947,6 +960,7 @@ global_filter_fn (GdkXEvent *gdk_xevent, GdkEvent *event, gpointer data)
 			      &= ~(XkbAX_StickyKeysFBMask);
 	        XkbChangeControls (display, priv->xkb_desc, &changes);
 	      }
+	      /* TODO: account for lock as well as latch */
 	      XkbLatchModifiers (display,
 				 XkbUseCoreKbd,
 				 priv->pending_xkb_mod_relatch_mask,
@@ -999,7 +1013,6 @@ spi_controller_register_with_devices (SpiDEController *controller)
 	  g_object_get_qdata (G_OBJECT (controller), spi_dec_private_quark);	 
   /* FIXME: should check for extension first! */
   XTestGrabControl (spi_get_display (), True);
-  priv->xkb_desc = XkbGetMap (spi_get_display (), 0, XkbUseCoreKbd);
 
   /* calls to device-specific implementations and routines go here */
   /* register with: keyboard hardware code handler */
@@ -1011,10 +1024,12 @@ spi_controller_register_with_devices (SpiDEController *controller)
 				      &priv->xkb_base_error_code, NULL, NULL);
   if (priv->have_xkb)
     {
+      priv->xkb_desc = XkbGetMap (spi_get_display (), 0, XkbUseCoreKbd);
       XkbSelectEvents (spi_get_display (),
 		       XkbUseCoreKbd,
 		       XkbStateNotifyMask, XkbStateNotifyMask);	    
     }	
+
   gdk_window_add_filter (NULL, global_filter_fn, controller);
 
   gdk_window_set_events (gdk_get_default_root_window (),
@@ -1212,9 +1227,10 @@ spi_keystroke_from_x_key_event (XKeyEvent *x_key_event)
   Accessibility_DeviceEvent key_event;
   KeySym keysym;
   const int cbuf_bytes = 20;
-  char cbuf [cbuf_bytes];
-  
-  keysym = XLookupKeysym (x_key_event, 0);
+  char cbuf [cbuf_bytes+1];
+  int nbytes;
+
+  nbytes = XLookupString (x_key_event, cbuf, cbuf_bytes, &keysym, NULL);  
   key_event.id = (CORBA_long)(keysym);
   key_event.hw_code = (CORBA_short) x_key_event->keycode;
   if (((XEvent *) x_key_event)->type == KeyPress)
@@ -1308,12 +1324,14 @@ spi_keystroke_from_x_key_event (XKeyEvent *x_key_event)
         key_event.event_string = CORBA_string_dup ("Right");
 	break;
       default:
-        if (XLookupString (x_key_event, cbuf, cbuf_bytes, &keysym, NULL) > 0)
+        if (nbytes > 0)
           {
+	    cbuf[nbytes] = '\0'; /* OK since length is cbuf_bytes+1 */
             key_event.event_string = CORBA_string_dup (cbuf);
-	    if (isgraph (keysym))
+	    if ((keysym && g_ascii_isprint (keysym)) || (nbytes == 1))
 	      {
-	        key_event.is_text = CORBA_TRUE; /* FIXME: incorrect for some composed chars? */
+	        key_event.is_text = CORBA_TRUE; 
+		/* FIXME: incorrect for some composed chars, unicode chars? */
 	      }
           }
         else
@@ -1789,11 +1807,8 @@ impl_generate_keyboard_event (PortableServer_Servant           servant,
 {
   SpiDEController *controller =
 	SPI_DEVICE_EVENT_CONTROLLER (bonobo_object (servant));
-  DEControllerPrivateData *priv;
   long key_synth_code;
-  unsigned int slow_keys_delay;
-  unsigned int press_time;
-  unsigned int release_time;
+  KeySym keysym;
 
 #ifdef SPI_DEBUG
 	fprintf (stderr, "synthesizing keystroke %ld, type %d\n",
@@ -1836,6 +1851,16 @@ impl_generate_keyboard_event (PortableServer_Servant           servant,
     {
       DBG (-1, g_warning ("Error emitting keystroke"));
     }
+  if (synth_type == Accessibility_KEY_SYM) {
+    keysym = keycode;
+  }
+  else {
+    keysym = XkbKeycodeToKeysym (spi_get_display (), keycode, 0, 0);
+  }
+  if (XkbKeysymToModifiers (spi_get_display (), keysym) == 0) 
+    {
+      spi_dec_clear_unlatch_pending (controller);
+    }
 }
 
 /* Accessibility::DEController::generateMouseEvent */
@@ -1846,7 +1871,7 @@ impl_generate_mouse_event (PortableServer_Servant servant,
 			   const CORBA_char      *eventName,
 			   CORBA_Environment     *ev)
 {
-  int button;
+  int button = 0;
   gboolean error = FALSE;
   Display *display = spi_get_display ();
 #ifdef SPI_DEBUG
@@ -1868,6 +1893,12 @@ impl_generate_mouse_event (PortableServer_Servant servant,
 	  case '3':
 	          button = 3;
 	          break;
+	  case '4':
+	          button = 4;
+	          break;
+	  case '5':
+	          button = 5;
+	          break;
 	  default:
 		  error = TRUE;
 	  }
@@ -1960,13 +1991,6 @@ spi_device_event_controller_init (SpiDEController *device_event_controller)
   device_event_controller->mouse_listeners = NULL;
   device_event_controller->keygrabs_list   = NULL;
 
-  /*
-   * TODO: fixme, this module makes the foolish assumptions that
-   * registryd uses the same display as the apps, and that the
-   * DISPLAY environment variable is set.
-   */
-  gdk_init (NULL, NULL);
-  
   private = g_new0 (DEControllerPrivateData, 1);
   gettimeofday (&private->last_press_time, NULL);
   gettimeofday (&private->last_release_time, NULL);
@@ -2011,7 +2035,6 @@ spi_device_event_controller_new (SpiRegistry *registry)
 {
   SpiDEController *retval = g_object_new (
     SPI_DEVICE_EVENT_CONTROLLER_TYPE, NULL);
-  DEControllerPrivateData *private;
   
   retval->registry = SPI_REGISTRY (bonobo_object_ref (
 	  BONOBO_OBJECT (registry)));
